{% extends "base.html" %}

{% block outdated %}
  You're not viewing the latest (stable) version.
  <a href="{{ '../' ~ base_url }}">
    <strong>Click here to go to latest (stable) version</strong>
  </a>
{% endblock %}
