# License: MIT
# Copyright © 2023 Frequenz Energy-as-a-Service GmbH

"""Test utilities."""

MSG_UNEXPECTED_REPO_TYPES = """\
Missing (or unexpected) repository types. Please make sure update:

- the documentation (in particular in src/frequenz/repo/config/__init__.py)
- the code (including all packages)
- the tests
"""
