/* Based on:
 * https://github.com/mkdocstrings/mkdocstrings/blob/master/docs/css/style.css
 */

/* Increase logo size */
.md-header__button.md-logo {
    padding-bottom: 0.2rem;
    padding-right: 0;
}
.md-header__button.md-logo img {
    height: 1.5rem;
}

/* Mark external links as such (also in nav) */
a.external:hover::after, a.md-nav__link[href^="https:"]:hover::after {
  /* https://primer.style/octicons/link-external-16 */
  background-image: url('data:image/svg+xml,<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 16 16"><path fill="rgb(233, 235, 252)" d="M10.604 1h4.146a.25.25 0 01.25.25v4.146a.25.25 0 01-.427.177L13.03 4.03 9.28 7.78a.75.75 0 01-1.06-1.06l3.75-3.75-1.543-1.543A.25.25 0 0110.604 1zM3.75 2A1.75 1.75 0 002 3.75v8.5c0 .966.784 1.75 1.75 1.75h8.5A1.75 1.75 0 0014 12.25v-3.5a.75.75 0 00-1.5 0v3.5a.25.25 0 01-.25.25h-8.5a.25.25 0 01-.25-.25v-8.5a.25.25 0 01.25-.25h3.5a.75.75 0 000-1.5h-3.5z"></path></svg>');
  height: 0.8em;
  width: 0.8em;
  margin-left: 0.2em;
  content: ' ';
  display: inline-block;
}

/* More space at the bottom of the page */
.md-main__inner {
  margin-bottom: 1.5rem;
}
